@@ -47,8 +47,8 @@ cmd_args(struct task_struct *task)
     if (task->pid == 0)
 	return("scheduler");
     
-    st_stk = task->mm[0].start_stack;
-    if ((n = vmread(buf, task->tss.cr3, task->mm[0].start_code + st_stk, STKBUF)) <= 0)
+    st_stk = task->mm->start_stack;
+    if ((n = vmread(buf, task->tss.cr3, task->mm->start_code + st_stk, STKBUF)) <= 0)
 	goto bad;
 
     argc = buf[0];
@@ -85,7 +85,7 @@ cmd_args(struct task_struct *task)
 	 * value of argv[0] (prog may have incremented argv[0])
 	 */
 	st_stk = argv0 - 16;
-	n = vmread(buf, task->tss.cr3, task->mm[0].start_code + st_stk, maxcmd+16);
+	n = vmread(buf, task->tss.cr3, task->mm->start_code + st_stk, maxcmd+16);
 	if ((n -= 16) <= 0)
 	    goto bad;
     } else {
