@@ -18,7 +18,6 @@
 #include <time.h>
 #include <sys/ioctl.h>
 #include <linux/sched.h>
-#include <linux/tty.h>
 #include <linux/ptrace.h>
 #include "ps.h"
 #include "psdata.h"
@@ -283,6 +282,7 @@ show_procs(void)
 {
     struct task_struct *taskp;
     struct task_struct task_buf;
+    struct tty_struct tty_buf;
     char stack_buf[PAGE_SIZE];
     char s[80];
     char *cmd;
@@ -297,14 +297,22 @@ show_procs(void)
 	kmemread(&taskp, _task + 4*i, 4);
 	if (taskp) {
 	    kmemread(&task_buf, (unsigned long) taskp, sizeof(task_buf));
-			/* check if valid, proc may have exited */
+
+	    /* check if valid, proc may have exited */
 	    if ((unsigned) task_buf.state > 5 ||
 		    (task_buf.pid <= 0 && i != 0) ||
 		    !task_buf.kernel_stack_page)
 		continue;
 
+	    /* dereference task sub-structures into our address space */
+	    if (task_buf.tty) {
+		kmemread(&tty_buf, (unsigned long) task_buf.tty,
+			 sizeof(tty_buf));
+		task_buf.tty = &tty_buf;
+	    }
+
 	    if (task_buf.pid == 0)
-		task_buf.mm[0].rss = 0;
+		task_buf.mm->rss = 0;
 
 	    if (npid) {
 		for (j = 0; j < npid; ++j)
@@ -458,7 +466,7 @@ show_long(char *s, struct task_struct *task, reg_t *stack)
 	PZERO - task->priority,		/* nice value */
 #endif
 	VSIZE(task,stack),
-	task->mm[0].rss * 4,
+	task->mm->rss * 4,
 	(task->state == TASK_INTERRUPTIBLE ||
 	 task->state == TASK_UNINTERRUPTIBLE ||
 	 (Debug > 1 && task->state == TASK_STOPPED) ? 
@@ -476,7 +484,7 @@ show_jobs(char *s, struct task_struct *task, reg_t *stack)
 	task->pgrp,
 	task->session,
 	dev_to_tty(TASK_TTY(task)),
-	tty_pgrp(TASK_TTY(task),task->tty),
+	TASK_TTY_PGRP(task),
 	status(task),
 	task->euid);
 }
@@ -502,14 +510,14 @@ show_user(char *s, struct task_struct *task, reg_t *stack)
 	pcpu = (double) (task->utime + task->stime) * 1000 / proc_jiffies;
     if (pcpu >= 1000)
 	pcpu = 999;
-    pmem = task->mm[0].rss * 1000 / nr_pages;
+    pmem = task->mm->rss * 1000 / nr_pages;
 
     sprintf(s, "%5d %2d.%d %2d.%d %4ld %4ld %-7s %-5s%.6s ",
 	task->pid,
 	pcpu / 10, pcpu % 10,
 	pmem / 10, pmem % 10,
 	VSIZE(task,stack),
-	task->mm[0].rss * 4,
+	task->mm->rss * 4,
 	dev_to_tty(TASK_TTY(task)),
 	status(task),
  	ctime(&start) + (time_now - start > 3600*24 ? 4 : 10));
@@ -588,14 +596,14 @@ show_vm(char *s, struct task_struct *task, reg_t *stack)
     sprint_time(s, task);
     s += strlen(s);
     s += sprintf(s, " %6ld %4ld %4ld %4ld ",
-	task->mm[0].maj_flt + (Sum ? task->mm[0].cmaj_flt : 0),
-	task->mm[0].end_code / 1024,
-	SIZE(task, stack), task->mm[0].rss*4);
+	task->mm->maj_flt + (Sum ? task->mm->cmaj_flt : 0),
+	task->mm->end_code / 1024,
+	SIZE(task, stack), task->mm->rss*4);
     if (task->rlim[RLIMIT_RSS].rlim_cur == RLIM_INFINITY)
 	s += sprintf(s, "   xx ");
     else
 	s += sprintf(s, "%5d ", task->rlim[RLIMIT_RSS].rlim_cur / 1024);
-    pmem = task->mm[0].rss * 1000 / nr_pages;
+    pmem = task->mm->rss * 1000 / nr_pages;
     sprintf(s, "%2d.%d", pmem / 10, pmem % 10);
 }
 
@@ -608,8 +616,8 @@ show_m(char *s, struct task_struct *task, reg_t *stack)
     sprintf(s, "%5d %-7s %5ld %5ld %4d %4d %4d %4d %4d %4d %4d %3d", 
 	task->pid,
 	dev_to_tty(TASK_TTY(task)),
-	task->mm[0].maj_flt + (Sum ? task->mm[0].cmaj_flt : 0),
-	task->mm[0].min_flt + (Sum ? task->mm[0].cmin_flt : 0),
+	task->mm->maj_flt + (Sum ? task->mm->cmaj_flt : 0),
+	task->mm->min_flt + (Sum ? task->mm->cmin_flt : 0),
 	/*task->end_code / 4096 << pg_shift,*/
 	mi->trs << pg_shift,
 	mi->drs << pg_shift,
@@ -652,9 +660,9 @@ show_regs(char *s, struct task_struct *task, reg_t *stack)
 	task->flags,
 	task->pid,
 	/**
-	task->start_code >> 16,
+	task->mm->start_code >> 16,
 	**/
-	task->mm[0].start_stack,
+	task->mm->start_stack,
 	KSTK_ESP(stack),
 	KSTK_EIP(stack),
 	prtime(time1, task->timeout, jiffies),
