@@ -7,6 +7,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <sys/sysmacros.h>
 #include <linux/sched.h>
 #include <linux/tty.h>
 #include <linux/mm.h>
@@ -28,7 +29,7 @@ status(struct task_struct *task)
 	*p++ = 'I';
     else
 	*p++ = "RSDZTP" [task->state];
-    if (task->mm[0].rss == 0 && task->state != TASK_ZOMBIE && task->pid != 0)
+    if (task->mm->rss == 0 && task->state != TASK_ZOMBIE && task->pid != 0)
 	*p++ = 'W';
     if (task->priority > PZERO)
 	*p++ = '<';
@@ -38,7 +39,7 @@ status(struct task_struct *task)
 	*p++ = ((task->flags & PF_TRACESYS) ? 'X' : 'x');
     if (task->leader)
 	*p++ = 's';
-    if (TASK_TTY(task) != -1 && tty_pgrp(TASK_TTY(task),task->tty) == task->pgrp)
+    if (TASK_TTY(task) != -1 && TASK_TTY_PGRP(task) == task->pgrp)
 	*p++ = '+';
     *p = '\0';
     return(buf);
@@ -107,7 +108,7 @@ get_mem_info(struct task_struct *task)
 #endif
     unsigned long ptbl;
     int size=0;
-    int tpag = task->mm[0].end_code / PAGE_SIZE;
+    int tpag = task->mm->end_code / PAGE_SIZE;
     unsigned map_nr;
     static struct mem_info mi;
 
@@ -122,7 +123,7 @@ get_mem_info(struct task_struct *task)
 	return &mi;
     pagedir = KPTR(task->tss.cr3);
 #else
-    if (kmemread(pagedir, task->tss.cr3 + (task->mm[0].start_code >> 20),
+    if (kmemread(pagedir, task->tss.cr3 + (task->mm->start_code >> 20),
 	    sizeof pagedir) < 0)
 	return &mi;
 #endif
@@ -180,17 +181,3 @@ get_mem_info(struct task_struct *task)
     mi.size = size;
     return &mi;
 }
-
-int
-tty_pgrp(int tty, struct tty_struct *tt)
-{
-    static long tpgid[256];
-
-    if (tty == -1)
-	return -1;
-    tty = minor(tty);
-    if (tpgid[tty] == 0) {
-	kmemread(tpgid + tty, (unsigned long) &tt->pgrp, sizeof(tt->pgrp));
-    }
-    return tpgid[tty];
-}
