@@ -264,31 +264,29 @@ main(int argc, char *argv[])
 	}
 
 	while ((p = kvm_nextproc()) != NULL) {
-		long tpgid;
-
 		if (p->state == TASK_ZOMBIE || p->tty == NULL) {
 			free(p);
 			continue;
 		}
 		for (ep = ehead; ep != NULL; ep = ep->next) {
-			if (minor(ep->tdev) == minor(tty_kmemread(p->tty))) {
-				ep->jcpu += p->utime + p->stime +
-					p->cutime + p->cstime;
-				kmemread(&tpgid, (unsigned long) &p->tty->pgrp,
-					 sizeof(tpgid));
-				if (p->pgrp == tpgid) {
-					/*
-					 * Proc is in foreground of this terminal
-					 */
-					if (proc_compare(ep->proc, p)) {
-						if (ep->proc)
-							free(ep->proc);
-						ep->proc = p;
-						ep->pcpu = p->utime + p->stime;
-					}
-					break;
-				}
+			if (!p->tty)
+				continue;
+			if (ep->tdev != p->tty->device)
+				continue;
+			ep->jcpu += p->utime + p->stime +
+				p->cutime + p->cstime;
+			if (p->pgrp != p->tty->pgrp)
+				continue;
+			/*
+			 * Proc is in foreground of this terminal
+			 */
+			if (proc_compare(ep->proc, p)) {
+				if (ep->proc)
+					free(ep->proc);
+				ep->proc = p;
+				ep->pcpu = p->utime + p->stime;
 			}
+			break;
 		}
 	}
 	if ((ioctl(1, TIOCGWINSZ, &ws) == -1 &&
