@@ -5,6 +5,7 @@ kvm_nextproc(void)
 {
 
 	struct task_struct *task, *taskp;
+	struct tty_struct *tty;
 	static unsigned long _task;
 	static int n = NR_TASKS;
 	
@@ -26,10 +27,16 @@ repeat:
 
 	kmemread(task, (unsigned long) taskp, sizeof(struct task_struct));
 
-		    /* check if valid, proc may have exited */
+	/* check if valid, proc may have exited */
 	if ((unsigned) task->state > 4 || task->pid <= 0)
 		goto repeat;
 
+
+	if (!task->tty)
+		return(task);
+	tty = (struct tty_struct *) xmalloc(sizeof(struct tty_struct));
+	kmemread(tty, (unsigned long) task->tty, sizeof(struct tty_struct));
+	task->tty = tty;
 	return(task);
 }
 
